@@ -115,7 +115,31 @@ func()
     check(v.defined_methods, ["func"])
     check(v.unused_classes, ["Bar"])
     check(v.unused_funcs, [])
-    check(v.unused_methods, ["func"])
+    # Bar.func is unused, but it's hard to detect this without producing a
+    # false positive in test_function_and_method2.
+    check(v.unused_methods, [])
+
+
+def test_function_and_method2(v):
+    v.scan(
+        """\
+class Bar(object):
+    def func(self):
+        pass
+
+    other_name_for_func = func
+
+Bar().other_name_for_func()
+"""
+    )
+    check(v.defined_classes, ["Bar"])
+    check(v.defined_funcs, [])
+    check(v.defined_methods, ["func"])
+    check(v.defined_vars, ["other_name_for_func"])
+    check(v.unused_classes, [])
+    check(v.unused_funcs, [])
+    check(v.unused_methods, [])
+    check(v.unused_vars, [])
 
 
 def test_attribute1(v):
@@ -128,7 +152,7 @@ foo.bar = 2
     check(v.unused_funcs, [])
     check(v.defined_funcs, [])
     check(v.defined_attrs, ["bar", "bar"])
-    check(v.used_attrs, [])
+    check(v.used_names, ["foo"])
     check(v.unused_attrs, ["bar", "bar"])
 
 
@@ -143,11 +167,45 @@ A._d_ = 4
 """
     )
     check(v.defined_attrs, ["_", "_a", "__b", "__c__", "_d_"])
-    check(v.used_attrs, [])
+    check(v.used_names, ["A"])
     check(v.unused_attrs, ["_", "__b", "__c__", "_a", "_d_"])
     check(v.unused_vars, [])
 
 
+def test_getattr(v):
+    v.scan(
+        """\
+class Thing:
+    used_attr1 = 1
+    used_attr2 = 2
+    used_attr3 = 3
+    unused_attr = 4
+
+getattr(Thing, "used_attr1")
+getattr(Thing, "used_attr2", None)
+hasattr(Thing, "used_attr3")
+
+# Weird calls ignored
+hasattr(Thing, "unused_attr", None)
+getattr(Thing)
+getattr("unused_attr")
+getattr(Thing, "unused_attr", 1, 2)
+"""
+    )
+    check(v.unused_vars, ["unused_attr"])
+    check(
+        v.used_names,
+        [
+            "Thing",
+            "getattr",
+            "hasattr",
+            "used_attr1",
+            "used_attr2",
+            "used_attr3",
+        ],
+    )
+
+
 def test_callback1(v):
     v.scan(
         """\
@@ -159,7 +217,7 @@ b = Bar()
 b.foo
 """
     )
-    check(v.used_attrs, ["foo"])
+    check(v.used_names, ["Bar", "b", "foo"])
     check(v.defined_classes, ["Bar"])
     check(v.defined_funcs, [])
     check(v.defined_methods, ["foo"])
@@ -174,7 +232,7 @@ class Bar(object):
     pass
 """
     )
-    check(v.used_attrs, [])
+    check(v.used_names, [])
     check(v.defined_classes, ["Bar"])
     check(v.unused_classes, ["Bar"])
 
@@ -189,7 +247,6 @@ class Foo(Bar):
 Foo()
 """
     )
-    check(v.used_attrs, [])
     check(v.used_names, ["Bar", "Foo"])
     check(v.defined_classes, ["Bar", "Foo"])
     check(v.unused_classes, [])
@@ -203,7 +260,6 @@ class Bar():
 [Bar]
 """
     )
-    check(v.used_attrs, [])
     check(v.used_names, ["Bar"])
     check(v.defined_classes, ["Bar"])
     check(v.unused_classes, [])
@@ -217,7 +273,7 @@ class Bar():
 Bar()
 """
     )
-    check(v.used_attrs, [])
+    check(v.used_names, ["Bar"])
     check(v.defined_classes, ["Bar"])
     check(v.unused_classes, [])
 
@@ -230,7 +286,7 @@ class Bar():
 b = Bar()
 """
     )
-    check(v.used_attrs, [])
+    check(v.used_names, ["Bar"])
     check(v.defined_classes, ["Bar"])
     check(v.unused_classes, [])
     check(v.unused_vars, ["b"])
@@ -302,8 +358,7 @@ x.d
     )
     check(v.defined_funcs, [])
     check(v.defined_vars, ["b"])
-    check(v.used_names, ["a", "c", "x"])
-    check(v.used_attrs, ["d"])
+    check(v.used_names, ["a", "c", "d", "x"])
     check(v.unused_attrs, [])
     check(v.unused_funcs, [])
     check(v.unused_props, [])
@@ -322,7 +377,7 @@ def test_variable2(v):
     v.scan("a = 1\nc = b.a")
     check(v.defined_funcs, [])
     check(v.defined_vars, ["a", "c"])
-    check(v.used_names, ["b"])
+    check(v.used_names, ["a", "b"])
     check(v.unused_vars, ["c"])
 
 
@@ -411,7 +466,7 @@ class Bar(object):
     check(v.defined_attrs, ["a"])
     check(v.defined_classes, ["Bar"])
     check(v.defined_vars, [])
-    check(v.used_attrs, [])
+    check(v.used_names, [])
     check(v.unused_attrs, ["a"])
     check(v.unused_classes, ["Bar"])
 
@@ -434,13 +489,12 @@ class BasicTestCase:
 class OtherClass:
     pass
 """,
-        filename="test_function_names.py",
+        filename="dir/test_function_names.py",
     )
     check(v.defined_attrs, [])
     check(v.defined_classes, ["OtherClass"])
     check(v.defined_funcs, ["other_func"])
     check(v.defined_vars, [])
-    check(v.used_attrs, [])
     check(v.used_names, [])
     check(v.unused_attrs, [])
     check(v.unused_classes, ["OtherClass"])
@@ -456,7 +510,7 @@ async def test_func():
 async def other_func():
     pass
 """,
-        filename="test_function_names.py",
+        filename="dir/test_function_names.py",
     )
     check(v.defined_funcs, ["other_func"])
     check(v.unused_funcs, ["other_func"])
@@ -471,7 +525,7 @@ async def test_func():
 async def other_func():
     pass
 """,
-        filename="function_names.py",
+        filename="dir/function_names.py",
     )
     check(v.defined_funcs, ["test_func", "other_func"])
     check(v.unused_funcs, ["other_func", "test_func"])
@@ -500,7 +554,6 @@ class OtherClass:
     check(v.defined_classes, ["BasicTestCase", "OtherClass", "TestClass"])
     check(v.defined_funcs, ["test_func", "other_func"])
     check(v.defined_vars, [])
-    check(v.used_attrs, [])
     check(v.used_names, [])
     check(v.unused_attrs, [])
     check(v.unused_classes, ["BasicTestCase", "OtherClass", "TestClass"])
@@ -522,9 +575,8 @@ foo.a = 2
     )
     check(v.defined_attrs, ["a"])
     check(v.defined_vars, ["a"])
-    check(v.used_attrs, [])
     check(v.used_names, ["a", "foo"])
-    check(v.unused_attrs, ["a"])
+    check(v.unused_attrs, [])
 
 
 def test_boolean(v):
@@ -689,3 +741,21 @@ bad()
         assert not v.found_dead_code_or_error
     else:
         assert v.found_dead_code_or_error
+
+
+def test_unused_args_with_del(v):
+    v.scan(
+        """\
+def foo(a, b, c, d=3):
+    del c, d
+    return a + b
+
+foo(1, 2)
+"""
+    )
+
+    check(v.defined_funcs, ["foo"])
+    check(v.defined_vars, ["a", "b", "c", "d"])
+    check(v.used_names, ["foo", "a", "b", "c", "d"])
+    check(v.unused_vars, [])
+    check(v.unused_funcs, [])
